@@ -36,7 +36,7 @@ class Excel(object):
     def __init__(self, file_path, pre_existing_keys=None):
         self.file_path = file_path
         self.wb = openpyxl.load_workbook(
-            self.file_path, data_only=True, read_only=False, keep_vba=True
+            self.file_path, data_only=True, read_only=False, keep_vba=False
         )
         self.pre_existing_keys = pre_existing_keys
 
@@ -281,8 +281,10 @@ class Db(object):
             raise IOError(
                 f'0 table found in the database file! The input file: {self.file_path} may be unsupported or corrupted')
         if self.pre_existing_keys is not None:
+            # check for duplicates in tables_names,
+            # if table_names == None, check for duplcates for all the tables
             Debugger.check_for_duplicates(
-                self.pre_existing_keys, keys
+                self.pre_existing_keys, table_names or keys
             )
             # skip pre_existing_keys
         if table_names is not None:
